@@ -1,4 +1,3 @@
-use crate::iter::KeyIter;
 use crate::tx::{Tx, TxnEnv};
 pub use sanakirja::btree::page::Page;
 use sanakirja::btree::{BTreeMutPage, BTreePage, Iter, RevIter};
@@ -26,38 +25,41 @@ impl<
     K: ?Sized + Storable + PartialEq,
     V: ?Sized + Storable + PartialEq,
     P: BTreeMutPage<K, V> + BTreePage<K, V>,
-    RK: ?Sized + EncodeDecode<K>,
-    RV: ?Sized + EncodeDecode<V>,
+    RK: ?Sized + Encode<K>,
+    RV: ?Sized + Encode<V>,
   > DbPage<'a, K, V, P, RK, RV>
 {
-  pub fn put(&self, k: &K, v: &V) -> Result<bool, Error> {
+  pub fn put(&self, k: &RK, v: &RV) -> Result<bool, Error> {
     db_page_w!(self, db, db.put(k, v))
   }
 
-  pub fn rm(&self, k: &K) -> Result<usize, Error> {
+  pub fn rm(&self, k: &RK) -> Result<usize, Error> {
     db_page_w!(self, db, db.rm(k))
   }
 
-  pub fn one(&self, k: &'a K) -> Result<Option<&'a V>, <TxnEnv as LoadPage>::Error> {
+  pub fn one(&self, k: &'a RK) -> Result<Option<&'a V>, <TxnEnv as LoadPage>::Error> {
     db_page_r!(self, db, db.one(k))
   }
 
-  pub fn exist(&self, k: &K, v: &V) -> Result<bool, <TxnEnv as LoadPage>::Error> {
+  pub fn exist(&self, k: &RK, v: &RV) -> Result<bool, <TxnEnv as LoadPage>::Error> {
     db_page_r!(self, db, db.exist(k, v))
   }
 
-  pub fn rm1<IntoV: Into<Option<&'a V>>>(&self, k: &K, v: IntoV) -> Result<bool, Error> {
+  pub fn rm1<IntoV: Into<Option<&'a RV>>>(&self, k: &RK, v: IntoV) -> Result<bool, Error> {
     db_page_w!(self, db, db.rm1(k, v.into()))
   }
 
   pub fn key_iter(
     &self,
-    k: &'a K,
-  ) -> Result<KeyIter<TxnEnv, K, V, P>, <TxnEnv as LoadPage>::Error> {
+    k: &'a RK,
+  ) -> Result<
+    Box<dyn Iterator<Item = Result<(&'a K, &'a V), <TxnEnv as LoadPage>::Error>> + 'a>,
+    <TxnEnv as LoadPage>::Error,
+  > {
     db_page_r!(self, db, db.key_iter(k))
   }
 
-  pub fn iter<OptionK: Into<Option<&'a K>>, OptionV: Into<Option<&'a V>>>(
+  pub fn iter<OptionK: Into<Option<&'a RK>>, OptionV: Into<Option<&'a RV>>>(
     &self,
     k: OptionK,
     v: OptionV,
@@ -65,7 +67,7 @@ impl<
     db_page_r!(self, db, db.iter(k.into(), v.into()))
   }
 
-  pub fn riter<OptionK: Into<Option<&'a K>>, OptionV: Into<Option<&'a V>>>(
+  pub fn riter<OptionK: Into<Option<&'a RK>>, OptionV: Into<Option<&'a RV>>>(
     &self,
     k: OptionK,
     v: OptionV,
@@ -74,31 +76,63 @@ impl<
   }
 }
 
-pub trait EncodeDecode<T: ?Sized> {
-  /*
-  fn encode(&self) -> &T;
-  fn decode(val: &T) -> Self;
-  */
-}
-
-impl EncodeDecode<u64> for u64 {}
-
-impl EncodeDecode<[u8]> for [u8] {
-  /*
-    fn encode(&self) -> &[u8]{self};
-    fn decode(val: &[u8]) -> [u8]{*val};
-  */
-}
-
 pub struct DbPage<
   'a,
   K: ?Sized + Storable + PartialEq,
   V: ?Sized + Storable + PartialEq,
   P: BTreeMutPage<K, V> + BTreePage<K, V>,
-  RK: ?Sized + EncodeDecode<K>,
-  RV: ?Sized + EncodeDecode<V>,
+  RK: ?Sized + Encode<K>,
+  RV: ?Sized + Encode<V>,
 > {
   pub(crate) tx: &'a Tx,
   pub id: usize,
   pub(crate) _kvp: PhantomData<(&'a K, &'a V, &'a P, &'a RK, &'a RV)>,
 }
+
+#[cfg(feature = "desse")]
+pub trait Encode<T: ?Sized> {
+  fn encode<R: Sized>(&self, next: &mut dyn FnMut(&T) -> R) -> R;
+}
+
+#[cfg(feature = "desse")]
+#[macro_export]
+macro_rules! encode {
+  ($cls:ty, $t:ty) => {
+    impl Encode<$t> for $cls {
+      #[inline]
+      fn encode<R: Sized>(&self, next: &mut dyn FnMut(&$t) -> R) -> R {
+        next(self)
+      }
+    }
+  };
+  ($cls:ty) => {
+    encode!($cls, $cls);
+  };
+}
+
+#[cfg(feature = "desse")]
+macro_rules! encode_li {
+  ( $( $x:ty ),* ) => {
+    $(encode!($x);)*
+  };
+}
+
+#[cfg(feature = "desse")]
+encode_li!(
+  [u8],
+  bool,
+  i8,
+  u8,
+  i16,
+  u16,
+  i32,
+  u32,
+  i64,
+  u64,
+  i128,
+  u128,
+  isize,
+  usize,
+  f32,
+  f64
+);
