@@ -1,13 +1,13 @@
 mod dbpage;
-pub use dbpage::{DbPage, EncodeDecode};
+pub use dbpage::{DbPage, Encode};
 mod tx;
 pub use tx::{MutTxnEnv, Tx, TxnEnv};
 mod iter;
-use iter::{key_iter, KeyIter};
+use iter::key_iter;
 
 pub use sanakirja::btree::page::Page;
 use sanakirja::btree::{create_db_, BTreeMutPage, BTreePage, Db_, Iter, RevIter};
-pub use sanakirja::{btree, Commit, Error, Storable, UnsizedStorable};
+pub use sanakirja::{btree, direct_repr, Commit, Error, Storable, UnsizedStorable};
 use sanakirja::{Env, LoadPage, RootDb};
 use std::convert::Into;
 use std::fs::create_dir_all;
@@ -21,29 +21,62 @@ use std::result::Result;
 macro_rules! sdb {
   ($cls:ident) => {
     sanakirja::direct_repr!($cls);
-    impl sdb::EncodeDecode<$cls> for $cls {}
+    sdb::encode!($cls);
   };
 }
 
 pub struct TxDb<
+  'a,
   K: Storable + PartialEq + ?Sized,
   V: Storable + PartialEq + ?Sized,
   T: LoadPage,
   P: BTreeMutPage<K, V> + BTreePage<K, V>,
+  RK: ?Sized + Encode<K>,
+  RV: ?Sized + Encode<V>,
 > {
   db: Db_<K, V, P>,
   id: usize,
   tx: *mut T,
+  pub(crate) _rkv: PhantomData<(&'a RK, &'a RV)>,
 }
 
 type UP<K, V> = btree::page_unsized::Page<K, V>;
 
 pub type Db<'a, K, V> = DbPage<'a, K, V, Page<K, V>, K, V>;
 pub type DbU<'a, K, V> = DbPage<'a, K, V, UP<K, V>, K, V>;
+pub type DbEk<'a, K, V, RK> = DbPage<'a, K, V, UP<K, V>, RK, V>;
+pub type DbEv<'a, K, V, RV> = DbPage<'a, K, V, UP<K, V>, K, RV>;
+pub type DbEkv<'a, K, V, RK, RV> = DbPage<'a, K, V, UP<K, V>, RK, RV>;
 
 pub struct WriteTx<'a>(ManuallyDrop<MutTxnEnv<'a>>);
 pub struct ReadTx<'a>(TxnEnv<'a>);
 
+#[cfg(feature = "desse")]
+#[macro_export]
+macro_rules! desse {
+  ($cls:ident, $desse:ident) => {
+    #[derive(
+      Default, Eq, PartialEq, PartialOrd, Ord, Hash, Clone, Copy, Debug, DesseSized, Desse,
+    )]
+    pub struct $desse([u8; $cls::SIZE]);
+
+    sdb::direct_repr!($desse);
+
+    impl Encode<$desse> for $cls {
+      #[inline]
+      fn encode<R: Sized>(&self, next: &mut dyn FnMut(&$desse) -> R) -> R {
+        next(&$desse(self.serialize()))
+      }
+    }
+
+    impl From<&$desse> for $cls {
+      fn from(v: &$desse) -> Self {
+        Data2::deserialize_from(&v.0)
+      }
+    }
+  };
+}
+
 macro_rules! tx {
   ($cls:ident, $tx:tt) => {
     impl<'a> $cls<'a> {
@@ -51,16 +84,17 @@ macro_rules! tx {
         K: ?Sized + Storable + PartialEq,
         V: ?Sized + Storable + PartialEq,
         P: BTreeMutPage<K, V> + BTreePage<K, V>,
-        RK: ?Sized + EncodeDecode<K>,
-        RV: ?Sized + EncodeDecode<V>,
+        RK: ?Sized + Encode<K>,
+        RV: ?Sized + Encode<V>,
       >(
         &self,
         db: &DbPage<K, V, P, RK, RV>,
-      ) -> TxDb<K, V, $tx<'a>, P> {
+      ) -> TxDb<K, V, $tx<'a>, P, RK, RV> {
         TxDb {
           id: db.id,
           db: self.btree(db.id),
           tx: self.ptr() as *mut $tx,
+          _rkv: PhantomData {},
         }
       }
     }
@@ -131,25 +165,46 @@ impl<'a> Drop for WriteTx<'a> {
   }
 }
 
+macro_rules! encode {
+  ($i:ident, $fn:expr) => {
+    $i.encode(&mut |$i| $fn)
+  };
+}
+
+macro_rules! encode_k_v {
+  ($k:ident, $v:ident, $fn:expr) => {
+    encode!($k, encode!($v, $fn))
+  };
+}
+
 macro_rules! iter {
   ($cls:ident, $fn:ident, $real:expr) => {
-    pub fn $fn<OptionK: Into<Option<&'a K>>, OptionV: Into<Option<&'a V>>>(
+    #[inline]
+    pub fn $fn<OptionK: Into<Option<&'a RK>>, OptionV: Into<Option<&'a RV>>>(
       &self,
-      key: OptionK,
-      value: OptionV,
+      k: OptionK,
+      v: OptionV,
     ) -> Result<$cls<'a, T, K, V, P>, <T as LoadPage>::Error> {
-      let tx = unsafe { &*self.tx };
-      $real(
-        tx,
-        &self.db,
-        match key.into() {
-          None => None,
-          Some(k) => match value.into() {
-            None => Some((k, None)),
-            Some(v) => Some((k, Some(v))),
-          },
+      match k.into() {
+        None => {
+          let tx = unsafe { &*self.tx };
+          $real(tx, &self.db, None)
+        }
+        Some(k) => match v.into() {
+          Some(v) => {
+            encode_k_v!(k, v, {
+              let tx = unsafe { &*self.tx };
+              $real(tx, &self.db, Some((k, Some(v))))
+            })
+          }
+          None => {
+            encode!(k, {
+              let tx = unsafe { &*self.tx };
+              $real(tx, &self.db, Some((k, None)))
+            })
+          }
         },
-      )
+      }
     }
   };
 }
@@ -157,50 +212,63 @@ macro_rules! iter {
 // all TxDb
 impl<
     'a,
+    'b,
     K: 'a + PartialEq + Storable + ?Sized,
     V: 'a + PartialEq + Storable + ?Sized,
     T: 'a + LoadPage,
-    P: BTreeMutPage<K, V> + BTreePage<K, V>,
-  > TxDb<K, V, T, P>
+    P: 'a + BTreeMutPage<K, V> + BTreePage<K, V>,
+    RK: 'a + ?Sized + Encode<K>,
+    RV: 'a + ?Sized + Encode<V>,
+  > TxDb<'b, K, V, T, P, RK, RV>
 {
   iter!(Iter, iter, btree::iter);
   iter!(RevIter, riter, btree::rev_iter);
 
-  pub fn key_iter(&self, key: &'a K) -> Result<KeyIter<'a, T, K, V, P>, T::Error> {
+  #[inline]
+  pub fn key_iter<'c>(
+    &self,
+    k: &'a RK,
+  ) -> Result<Box<dyn Iterator<Item = Result<(&'a K, &'a V), T::Error>> + 'a>, T::Error> {
     let tx = unsafe { &*self.tx };
-    key_iter(tx, &self.db, key)
+    encode!(k, key_iter(tx, &self.db, k))
   }
 
-  pub fn exist(&self, k: &K, v: &V) -> Result<bool, <T as LoadPage>::Error> {
-    let tx = unsafe { &*self.tx };
-    match btree::get(tx, &self.db, k, Some(v))? {
-      None => Ok(false),
-      Some((key, val)) => {
-        if key == k {
-          if val == v {
-            Ok(true)
+  #[inline]
+  pub fn exist(&self, k: &RK, v: &RV) -> Result<bool, <T as LoadPage>::Error> {
+    encode_k_v!(k, v, {
+      let tx = unsafe { &*self.tx };
+      match btree::get(tx, &self.db, k, Some(v))? {
+        None => Ok(false),
+        Some((key, val)) => {
+          if key == k {
+            if val == v {
+              Ok(true)
+            } else {
+              Ok(false)
+            }
           } else {
             Ok(false)
           }
-        } else {
-          Ok(false)
         }
       }
-    }
+    })
   }
 
-  pub fn one(&self, k: &K) -> Result<Option<&'a V>, <T as LoadPage>::Error> {
-    let tx = unsafe { &*self.tx };
-    match btree::get(tx, &self.db, k, None)? {
-      None => Ok(None),
-      Some((key, v)) => {
-        if key == k {
-          Ok(Some(v))
-        } else {
-          Ok(None)
+  #[inline]
+  pub fn one(&self, k: &RK) -> Result<Option<&'a V>, <T as LoadPage>::Error> {
+    encode!(k, {
+      let tx = unsafe { &*self.tx };
+      match btree::get(tx, &self.db, k, None)? {
+        None => Ok(None),
+        Some((key, v)) => {
+          if key == k {
+            Ok(Some(v))
+          } else {
+            Ok(None)
+          }
         }
       }
-    }
+    })
   }
 }
 
@@ -220,31 +288,58 @@ macro_rules! set_root {
 // write tx TxDb
 impl<
     'a,
+    'b,
     K: 'a + Storable + PartialEq + ?Sized,
     V: 'a + Storable + PartialEq + ?Sized,
     P: BTreeMutPage<K, V> + BTreePage<K, V>,
-  > TxDb<K, V, MutTxnEnv<'a>, P>
+    RK: ?Sized + Encode<K>,
+    RV: ?Sized + Encode<V>,
+  > TxDb<'b, K, V, MutTxnEnv<'b>, P, RK, RV>
 {
-  pub fn put(&mut self, k: &K, v: &V) -> std::result::Result<bool, Error> {
-    set_root!(btree::put(tx, &mut self.db, k, v), self, tx)
+  #[inline]
+  pub fn put(&mut self, k: &RK, v: &RV) -> std::result::Result<bool, Error> {
+    encode_k_v!(
+      k,
+      v,
+      set_root!(btree::put(tx, &mut self.db, k, v), self, tx)
+    )
   }
 
-  pub fn rm1<IntoV: Into<Option<&'a V>>>(&mut self, k: &K, v: IntoV) -> Result<bool, Error> {
-    set_root!(btree::del(tx, &mut self.db, k, v.into()), self, tx)
+  #[inline]
+  pub fn rm1<IntoV: Into<Option<&'b RV>>>(&mut self, k: &RK, v: IntoV) -> Result<bool, Error> {
+    match v.into() {
+      Some(v) => {
+        encode_k_v!(
+          k,
+          v,
+          set_root!(btree::del(tx, &mut self.db, k, Some(v)), self, tx)
+        )
+      }
+      None => {
+        encode!(
+          k,
+          set_root!(btree::del(tx, &mut self.db, k, None), self, tx)
+        )
+      }
+    }
   }
 
-  pub fn rm(&mut self, k: &K) -> Result<usize, Error> {
-    set_root!(
-      {
-        let db = &mut self.db;
-        let mut n = 0usize;
-        while btree::del(tx, db, k, None)? {
-          n += 1
-        }
-        Ok(n)
-      },
-      self,
-      tx
+  #[inline]
+  pub fn rm(&mut self, k: &RK) -> Result<usize, Error> {
+    encode!(
+      k,
+      set_root!(
+        {
+          let db = &mut self.db;
+          let mut n = 0usize;
+          while btree::del(tx, db, k, None)? {
+            n += 1
+          }
+          Ok(n)
+        },
+        self,
+        tx
+      )
     )
   }
 }
@@ -268,8 +363,8 @@ impl Tx {
     K: ?Sized + Storable + PartialEq,
     V: ?Sized + Storable + PartialEq,
     P: BTreeMutPage<K, V> + BTreePage<K, V>,
-    RK: ?Sized + EncodeDecode<K>,
-    RV: ?Sized + EncodeDecode<V>,
+    RK: ?Sized + Encode<K>,
+    RV: ?Sized + Encode<V>,
   >(
     &self,
     id: usize,
