@@ -1,6 +1,6 @@
 mod db;
 use anyhow::Result;
-use db::{Data, Hash, DB0, DB1, DB2, DB3, DB4, TX};
+use db::{Data, Data2, Hash, DB0, DB1, DB2, DB3, DB4, DB5, TX};
 use sdb::UnsizedStorable;
 
 #[test]
@@ -29,6 +29,8 @@ fn main() -> Result<()> {
     db0.put(&2, &1)?;
     db0.put(&3, &7)?;
     db0.put(&3, &9)?;
+    db0.put(&5, &1)?;
+    db0.put(&5, &9)?;
 
     println!("- print all key db0");
     for entry in db0.iter(None, None)? {
@@ -37,7 +39,7 @@ fn main() -> Result<()> {
     }
 
     println!("- print db1 where key is 2");
-    for entry in db0.key_iter(&2)? {
+    for entry in db0.key_iter(&4)? {
       let (k, v) = entry?;
       println!("> {:?} {:?}", k, v)
     }
@@ -93,17 +95,29 @@ fn main() -> Result<()> {
       println!("> {:?} {:?}", k, v)
     }
 
+    let mut db5 = tx.db(&DB5);
+    let data = Data2 {
+      id: 1234,
+      hash: [3, 2, 1],
+    };
+    db5.put(&1, &data)?;
+    println!("- print all key db5");
+    for entry in db5.iter(None, None)? {
+      let (k, v) = entry?;
+      println!("> {:?} {:?}", k, Data2::from(v))
+    }
     //write tx will auto commit when drop
   }
+
   {
     println!("# read transaction");
 
     let tx = TX.r()?; //
     let db0 = tx.db(&DB0);
 
-    println!("- exist key 2 value 1 > {:?}", db0.exist(&2, &1)?);
+    dbg!(db0.exist(&2, &1)?);
     for i in [1, 2, 5] {
-      println!("- get key {} > {:?}", i, db0.one(&i)?);
+      dbg!(i, db0.one(&i)?);
     }
 
     println!("- print all key");
